@@ -2,7 +2,6 @@ package oviewer
 
 import (
 	"bufio"
-	"bytes"
 	"context"
 	"errors"
 	"fmt"
@@ -11,7 +10,6 @@ import (
 	"os"
 	"strings"
 	"sync/atomic"
-	"time"
 
 	"golang.org/x/term"
 )
@@ -20,6 +18,9 @@ import (
 // This bufSize is used when only counting.
 const bufSize = 4096
 
+// tailSize represents the position to start reading backwards from the end position.
+const tailSize = 10000
+
 // FormFeed is the delimiter that separates the sections.
 // The default delimiter that separates single output from watch.
 const FormFeed = "\f"
@@ -27,19 +28,39 @@ const FormFeed = "\f"
 // firstRead first reads the file.
 // Fill the contents of the read file into the first chunk.
 func (m *Document) firstRead(reader *bufio.Reader) (*bufio.Reader, error) {
-	atomic.StoreInt32(&m.noNewlineEOF, 0)
-	chunk := m.chunks[0]
-	if err := m.addLines(chunk, reader, 0); err != nil {
-		if !errors.Is(err, io.EOF) {
-			atomic.StoreInt32(&m.eof, 1)
-			return nil, err
+	atomic.StoreInt32(&m.store.noNewlineEOF, 0)
+	chunk := m.store.chunks[0]
+	if err := m.store.readLines(chunk, reader, 0, ChunkSize, true); err != nil {
+		if errors.Is(err, io.EOF) {
+			return m.afterEOF(reader), nil
 		}
-		reader = m.afterEOF(reader)
+		atomic.StoreInt32(&m.store.eof, 1)
+		return nil, err
 	}
 
-	if !m.BufEOF() {
-		m.requestContinue()
+	m.requestContinue()
+	return reader, nil
+}
+
+// tmpRead read tail to temporary store.
+// It is executed only once if EOF has not been reached after follow-mode is set.
+func (m *Document) tmpRead(reader *bufio.Reader) (*bufio.Reader, error) {
+	m.followStore = NewStore()
+	atomic.StoreInt32(&m.tmpFollow, 1)
+
+	if _, err := m.file.Seek(tailSize*-1, io.SeekEnd); err != nil {
+		return reader, fmt.Errorf("tmpFollowRead seek: %w", err)
 	}
+	reader.Reset(m.file)
+	chunk := m.followStore.chunks[0]
+	if err := m.followStore.readLines(chunk, reader, 0, ChunkSize, true); err != nil {
+		if !errors.Is(err, io.EOF) {
+			return nil, err
+		}
+	}
+
+	atomic.StoreInt32(&m.followStore.eof, 1)
+	m.requestContinue()
 	return reader, nil
 }
 
@@ -47,26 +68,24 @@ func (m *Document) firstRead(reader *bufio.Reader) (*bufio.Reader, error) {
 // and only reads the file or counts the lines of the file.
 func (m *Document) continueRead(reader *bufio.Reader) (*bufio.Reader, error) {
 	if m.seekable {
-		if _, err := m.file.Seek(m.offset, io.SeekStart); err != nil {
-			atomic.StoreInt32(&m.eof, 1)
-			log.Printf("seek: %s", err)
+		if err := m.seekChunk(reader, m.store.offset); err != nil {
+			atomic.StoreInt32(&m.store.eof, 1)
+			log.Printf("continueRead: %s", err)
 			m.seekable = false
 		} else {
 			reader.Reset(m.file)
 		}
 	}
-	chunk := m.chunkForAdd()
+	chunk := m.store.chunkForAdd(m.seekable, m.store.size)
 	start := len(chunk.lines)
-	if err := m.addChunk(chunk, reader, start); err != nil {
-		if !errors.Is(err, io.EOF) {
-			return nil, fmt.Errorf("addChunk: %w", err)
+	if err := m.addOrReserveChunk(chunk, reader, start, ChunkSize); err != nil {
+		if errors.Is(err, io.EOF) {
+			return m.afterEOF(reader), nil
 		}
-		reader = m.afterEOF(reader)
+		return nil, fmt.Errorf("addChunk: %w", err)
 	}
 
-	if !m.BufEOF() {
-		m.requestContinue()
-	}
+	m.requestContinue()
 	return reader, nil
 }
 
@@ -79,57 +98,96 @@ func (m *Document) followRead(reader *bufio.Reader) (*bufio.Reader, error) {
 		return reader, nil
 	}
 
-	reader, err := m.loadRead(reader, m.lastChunkNum())
+	reader, err := m.loadRead(reader, m.store.lastChunkNum())
 	if err != nil {
 		return reader, err
 	}
 
-	atomic.StoreInt32(&m.eof, 0)
-	chunk := m.chunks[m.lastChunkNum()]
+	atomic.StoreInt32(&m.store.eof, 0)
+	chunk := m.store.chunks[m.store.lastChunkNum()]
 	start := len(chunk.lines) - 1
-	if atomic.LoadInt32(&m.noNewlineEOF) == 0 {
-		chunk = m.chunkForAdd()
+	if atomic.LoadInt32(&m.store.noNewlineEOF) == 0 {
+		chunk = m.store.chunkForAdd(m.seekable, m.store.size)
 		start = len(chunk.lines)
 	}
 	if m.seekable {
-		if _, err := m.file.Seek(m.offset, io.SeekStart); err != nil {
-			return nil, fmt.Errorf("seek: %w", err)
+		if err := m.seekChunk(reader, m.store.offset); err != nil {
+			return nil, fmt.Errorf("followRead: %w", err)
 		}
 		reader = bufio.NewReader(m.file)
 	}
 
-	if err := m.addLines(chunk, reader, start); err != nil {
-		if !errors.Is(err, io.EOF) {
-			return nil, err
+	if err := m.store.readLines(chunk, reader, start, ChunkSize, true); err != nil {
+		if errors.Is(err, io.EOF) {
+			return m.afterEOF(reader), nil
 		}
-		reader = m.afterEOF(reader)
+		return nil, err
 	}
 
-	if !m.BufEOF() {
-		m.requestContinue()
+	m.requestContinue()
+	return reader, nil
+}
+
+// addOrReserveChunk reads a file to add or reserve a Chunk.
+// If it's a seekable file, it just reserves it for later reading.
+func (m *Document) addOrReserveChunk(chunk *chunk, reader *bufio.Reader, start int, end int) error {
+	if m.seekable {
+		return m.reserveChunk(reader, start, ChunkSize)
+	}
+	return m.store.readLines(chunk, reader, start, ChunkSize, true)
+}
+
+// reserveChunk reserves ChunkSize lines.
+// read and update size only.
+func (m *Document) reserveChunk(reader *bufio.Reader, start int, end int) error {
+	count, size, err := m.store.countLines(reader, start, end)
+	m.store.mu.Lock()
+	m.store.size += int64(size)
+	m.store.offset = m.store.size
+	m.store.mu.Unlock()
+	atomic.AddInt32(&m.store.endNum, int32(count))
+	atomic.StoreInt32(&m.store.changed, 1)
+	return err
+}
+
+// loadChunk actually loads the reserved Chunk.
+func (m *Document) loadChunk(reader *bufio.Reader, chunkNum int) (*bufio.Reader, error) {
+	chunk := m.store.chunks[chunkNum]
+	if err := m.seekChunk(reader, chunk.start); err != nil {
+		return nil, err
+	}
+
+	start, end := m.store.chunkRange(chunkNum)
+	if err := m.store.readLines(chunk, reader, start, end, false); err != nil {
+		log.Printf("Failed to read the expected number of lines(%d:%d): %s", start, end, err)
+		if errors.Is(err, io.EOF) {
+			return m.afterEOF(reader), nil
+		}
+		return nil, err
 	}
 	return reader, nil
 }
 
+// seekChunk seeks to the start of the chunk.
+func (m *Document) seekChunk(reader *bufio.Reader, start int64) error {
+	if _, err := m.file.Seek(start, io.SeekStart); err != nil {
+		return fmt.Errorf("seek: %w", err)
+	}
+	reader.Reset(m.file)
+	return nil
+}
+
 // searchRead searches chunks and loads chunks if found.
 func (m *Document) searchRead(reader *bufio.Reader, chunkNum int, searcher Searcher) (*bufio.Reader, error) {
 	if _, err := m.searchChunk(chunkNum, searcher); err != nil {
-		if errors.Is(err, ErrNotFound) {
-			return reader, nil
-		}
 		return reader, err
 	}
-	if err := m.evictChunksFile(chunkNum); err != nil {
-		log.Println(err)
-		return reader, nil
-	}
-	return m.readChunk(reader, chunkNum)
+	return m.loadReadFile(reader, chunkNum)
 }
 
 // loadRead loads the read contents into chunks.
 func (m *Document) loadRead(reader *bufio.Reader, chunkNum int) (*bufio.Reader, error) {
 	if m.seekable {
-		m.currentChunk = chunkNum
 		return m.loadReadFile(reader, chunkNum)
 	}
 	return m.loadReadMem(reader, chunkNum)
@@ -138,13 +196,12 @@ func (m *Document) loadRead(reader *bufio.Reader, chunkNum int) (*bufio.Reader,
 // loadReadFile loads the read contents into chunks.
 // loadReadFile frees old chunks and loads new chunks.
 func (m *Document) loadReadFile(reader *bufio.Reader, chunkNum int) (*bufio.Reader, error) {
-	_ = m.evictChunksFile(chunkNum)
-	m.addChunksFile(chunkNum)
-	if len(m.chunks[chunkNum].lines) != 0 {
+	m.store.swapLoadedFile(chunkNum)
+	if len(m.store.chunks[chunkNum].lines) != 0 {
 		// already loaded.
 		return reader, nil
 	}
-	return m.readChunk(reader, chunkNum)
+	return m.loadChunk(reader, chunkNum)
 }
 
 // loadReadMem loads the read contents into chunks.
@@ -154,61 +211,27 @@ func (m *Document) loadReadMem(reader *bufio.Reader, chunkNum int) (*bufio.Reade
 		return reader, nil
 	}
 
-	if chunkNum < m.lastChunkNum() {
+	if chunkNum < m.store.lastChunkNum() {
 		// already loaded.
 		// return reader, fmt.Errorf("%w %d", ErrAlreadyLoaded, chunkNum)
 		return reader, nil
 	}
-	m.evictChunksMem(chunkNum)
+	m.store.evictChunksMem(chunkNum)
 	m.requestContinue()
 	return reader, nil
 }
 
-// readChunk loads the read contents into chunks.
-func (m *Document) readChunk(reader *bufio.Reader, chunkNum int) (*bufio.Reader, error) {
-	chunk := m.chunks[chunkNum]
-	if _, err := m.file.Seek(chunk.start, io.SeekStart); err != nil {
-		return nil, fmt.Errorf("seek: %w", err)
-	}
-	reader.Reset(m.file)
-	start := 0
-	lastChunk, end := chunkLine(m.endNum)
-	if chunkNum != lastChunk {
-		end = ChunkSize
-	}
-	// log.Println("fillChunk", chunkNum, start, end)
-	if err := m.fillChunk(chunk, reader, start, end); err != nil {
-		if !errors.Is(err, io.EOF) {
-			return nil, err
-		}
-		reader = m.afterEOF(reader)
-	}
-	return reader, nil
-}
-
 // reloadRead performs reload processing.
 func (m *Document) reloadRead(reader *bufio.Reader) (*bufio.Reader, error) {
 	if !m.WatchMode {
 		m.reset()
 	} else {
 		m.seekable = false
-		chunk := m.chunkForAdd()
-		m.appendFormFeed(chunk)
+		chunk := m.store.chunkForAdd(m.seekable, m.store.size)
+		m.store.appendFormFeed(chunk)
 	}
-	var err error
-	reader, err = m.reloadFile(reader)
-	if err != nil {
-		return nil, err
-	}
-	return reader, nil
-}
 
-// addChunk fills or reserves a chunk.
-func (m *Document) addChunk(chunk *chunk, reader *bufio.Reader, start int) error {
-	if m.seekable {
-		return m.reserveChunk(reader, start)
-	}
-	return m.addLines(chunk, reader, start)
+	return m.reloadFile(reader)
 }
 
 // reloadFile reloads a file.
@@ -225,7 +248,7 @@ func (m *Document) reloadFile(reader *bufio.Reader) (*bufio.Reader, error) {
 	m.ClearCache()
 
 	atomic.StoreInt32(&m.closed, 0)
-	atomic.StoreInt32(&m.eof, 0)
+	atomic.StoreInt32(&m.store.eof, 0)
 	r, err := m.openFileReader(m.FileName)
 	if err != nil {
 		str := fmt.Sprintf("Access is no longer possible: %s", err)
@@ -238,8 +261,11 @@ func (m *Document) reloadFile(reader *bufio.Reader) (*bufio.Reader, error) {
 
 // afterEOF does processing after reaching EOF.
 func (m *Document) afterEOF(reader *bufio.Reader) *bufio.Reader {
-	m.offset = m.size
-	atomic.StoreInt32(&m.eof, 1)
+	m.store.offset = m.store.size
+	atomic.StoreInt32(&m.store.eof, 1)
+	if atomic.SwapInt32(&m.tmpFollow, 0) == 1 {
+		m.cache.Purge()
+	}
 	if !m.seekable { // for NamedPipe.
 		return bufio.NewReader(m.file)
 	}
@@ -253,22 +279,21 @@ func (m *Document) openFileReader(fileName string) (io.Reader, error) {
 	if err != nil {
 		return nil, err
 	}
-	r, err := m.fileReader(f)
-	if err != nil {
-		return nil, err
-	}
-	return r, nil
+	return m.fileReader(f)
 }
 
 // fileReader returns a io.Reader.
 func (m *Document) fileReader(f *os.File) (io.Reader, error) {
-	m.mu.Lock()
-	m.file = f
+	m.store.mu.Lock()
+	defer m.store.mu.Unlock()
 
+	atomic.StoreInt32(&m.closed, 0)
+	m.file = f
 	cFormat, r := uncompressedReader(m.file, m.seekable)
 	if cFormat == UNCOMPRESSED {
 		if m.seekable {
 			if _, err := f.Seek(0, io.SeekStart); err != nil {
+				atomic.StoreInt32(&m.closed, 1)
 				return nil, fmt.Errorf("seek: %w", err)
 			}
 			r = f
@@ -280,7 +305,6 @@ func (m *Document) fileReader(f *os.File) (io.Reader, error) {
 	if STDOUTPIPE != nil {
 		r = io.TeeReader(r, STDOUTPIPE)
 	}
-	m.mu.Unlock()
 
 	return r, nil
 }
@@ -302,214 +326,6 @@ func open(fileName string) (*os.File, error) {
 	return f, nil
 }
 
-// readAll reads to the end.
-// The read lines are stored in the lines of the Document.
-func (m *Document) readAll(reader *bufio.Reader) error {
-	chunk := m.chunkForAdd()
-	start := len(chunk.lines)
-	for {
-		if err := m.addLines(chunk, reader, start); err != nil {
-			return err
-		}
-		chunk = NewChunk(m.size)
-		m.mu.Lock()
-		m.chunks = append(m.chunks, chunk)
-		m.mu.Unlock()
-		start = 0
-	}
-}
-
-// fillChunk fills the chunk with lines read from reader.
-func (m *Document) fillChunk(chunk *chunk, reader *bufio.Reader, start int, end int) error {
-	return m.readLines(chunk, reader, start, end, false)
-}
-
-// addLines appends lines read from reader into chunks.
-func (m *Document) addLines(chunk *chunk, reader *bufio.Reader, start int) error {
-	return m.readLines(chunk, reader, start, ChunkSize, true)
-}
-
-// readLines append lines read from reader into chunks.
-// Read and fill the number of lines from start to end in chunk.
-// If isCount is true, increment the number of lines read (update endNum).
-func (m *Document) readLines(chunk *chunk, reader *bufio.Reader, start int, end int, addLines bool) error {
-	var line bytes.Buffer
-	var isPrefix bool
-	for num := start; num < end; {
-		if atomic.LoadInt32(&m.readCancel) == 1 {
-			break
-		}
-		buf, err := reader.ReadSlice('\n')
-		if errors.Is(err, bufio.ErrBufferFull) {
-			isPrefix = true
-			err = nil
-		}
-		line.Write(buf)
-		if isPrefix {
-			isPrefix = false
-			continue
-		}
-
-		num++
-		atomic.StoreInt32(&m.changed, 1)
-		if err != nil {
-			if line.Len() != 0 {
-				m.append(chunk, addLines, line.Bytes())
-				m.offset = m.size
-				atomic.StoreInt32(&m.noNewlineEOF, 1)
-			}
-			return err
-		}
-		m.append(chunk, addLines, line.Bytes())
-		m.offset = m.size
-		line.Reset()
-	}
-	return nil
-}
-
-// reserveChunk reserves ChunkSize lines.
-// read and update size only.
-func (m *Document) reserveChunk(reader *bufio.Reader, start int) error {
-	count, size, err := m.countLines(reader, start)
-	m.mu.Lock()
-	m.endNum += count
-	m.size += int64(size)
-	m.offset = m.size
-	m.mu.Unlock()
-	atomic.StoreInt32(&m.changed, 1)
-	return err
-}
-
-// countLines counts the number of lines and the size of the buffer.
-func (m *Document) countLines(reader *bufio.Reader, start int) (int, int, error) {
-	num := start
-	count := 0
-	size := 0
-	buf := make([]byte, bufSize)
-	for {
-		bufLen, err := reader.Read(buf)
-		if err != nil {
-			return count, size, fmt.Errorf("read: %w", err)
-		}
-		if bufLen == 0 {
-			return count, size, io.EOF
-		}
-
-		lSize := bufLen
-		lCount := bytes.Count(buf[:bufLen], []byte("\n"))
-		// If it exceeds ChunkSize, Re-aggregate size and count.
-		if num+lCount > ChunkSize {
-			lSize = 0
-			lCount = ChunkSize - num
-			for i := 0; i < lCount; i++ {
-				p := bytes.IndexByte(buf[lSize:bufLen], '\n')
-				lSize += p + 1
-			}
-		}
-
-		num += lCount
-		count += lCount
-		size += lSize
-		if num >= ChunkSize {
-			// no newline at the end of the file.
-			if bufLen < bufSize {
-				p := bytes.LastIndex(buf[:bufLen], []byte("\n"))
-				size -= bufLen - p - 1
-			}
-			break
-		}
-		// no newline at the end of the file.
-		if bufLen < bufSize {
-			p := bytes.LastIndex(buf[:bufLen], []byte("\n"))
-			if p+1 < bufLen {
-				count++
-				atomic.StoreInt32(&m.noNewlineEOF, 1)
-			}
-		}
-	}
-	return count, size, nil
-}
-
-// appendOnly appends to the line of the chunk.
-// appendOnly does not updates the number of lines and size.
-func (m *Document) appendOnly(chunk *chunk, line []byte) {
-	m.mu.Lock()
-	size := len(line)
-	dst := make([]byte, size)
-	copy(dst, line)
-	chunk.lines = append(chunk.lines, dst)
-	m.mu.Unlock()
-}
-
-// joinLast joins the new content to the last line.
-// This is used when the last line is added without a newline and EOF.
-func (m *Document) joinLast(chunk *chunk, line []byte) bool {
-	if len(chunk.lines) == 0 {
-		return false
-	}
-	size := len(line)
-
-	m.mu.Lock()
-	num := len(chunk.lines) - 1
-	buf := chunk.lines[num]
-	dst := make([]byte, 0, len(buf)+size)
-	dst = append(dst, buf...)
-	dst = append(dst, line...)
-	m.size += int64(size)
-	chunk.lines[num] = dst
-	m.mu.Unlock()
-
-	if line[len(line)-1] == '\n' {
-		atomic.StoreInt32(&m.noNewlineEOF, 0)
-	}
-	m.cache.Remove(m.endNum - 1)
-	return true
-}
-
-// append appends a line to the chunk.
-func (m *Document) append(chunk *chunk, isCount bool, line []byte) {
-	if isCount {
-		m.appendLine(chunk, line)
-		return
-	}
-
-	m.appendOnly(chunk, line)
-}
-
-// appendLine appends to the line of the chunk.
-// appendLine updates the number of lines and size.
-func (m *Document) appendLine(chunk *chunk, line []byte) {
-	if atomic.SwapInt32(&m.noNewlineEOF, 0) == 1 {
-		m.joinLast(chunk, line)
-		return
-	}
-	m.mu.Lock()
-	size := len(line)
-	m.size += int64(size)
-	m.endNum++
-	dst := make([]byte, size)
-	copy(dst, line)
-	chunk.lines = append(chunk.lines, dst)
-	m.mu.Unlock()
-}
-
-func (m *Document) appendFormFeed(chunk *chunk) {
-	line := ""
-	m.mu.Lock()
-	if len(chunk.lines) > 0 {
-		line = string(chunk.lines[len(chunk.lines)-1])
-	}
-	m.mu.Unlock()
-	// Do not add if the previous is FormFeed(always add for formfeedTime).
-	if line != FormFeed {
-		feed := FormFeed
-		if m.formfeedTime {
-			feed = fmt.Sprintf("%sTime: %s", FormFeed, time.Now().Format(time.RFC3339))
-		}
-		m.appendLine(chunk, []byte(feed))
-	}
-}
-
 // reload will read again.
 // Regular files are reopened and reread increase.
 // The pipe will reset what it has read.
@@ -523,9 +339,9 @@ func (m *Document) reload() error {
 		return ErrEOFreached
 	}
 
-	atomic.StoreInt32(&m.readCancel, 1)
+	atomic.StoreInt32(&m.store.readCancel, 1)
 	m.requestReload()
-	atomic.StoreInt32(&m.readCancel, 0)
+	atomic.StoreInt32(&m.store.readCancel, 0)
 	if !m.WatchMode {
 		m.topLN = 0
 	}
@@ -535,14 +351,9 @@ func (m *Document) reload() error {
 
 // reset clears all lines.
 func (m *Document) reset() {
-	m.mu.Lock()
-	m.size = 0
-	m.endNum = 0
-	m.chunks = []*chunk{
-		NewChunk(0),
-	}
-	m.mu.Unlock()
-	atomic.StoreInt32(&m.changed, 1)
+	m.store = NewStore()
+	m.store.setNewLoadChunks(m.seekable)
+	atomic.StoreInt32(&m.store.changed, 1)
 	m.ClearCache()
 }
 
@@ -557,13 +368,13 @@ func (m *Document) close() error {
 	if m.checkClose() {
 		return nil
 	}
-	log.Println("close")
+
 	if err := m.file.Close(); err != nil {
 		return fmt.Errorf("close: %w", err)
 	}
-	atomic.StoreInt32(&m.eof, 1)
+	atomic.StoreInt32(&m.store.eof, 1)
 	atomic.StoreInt32(&m.closed, 1)
-	atomic.StoreInt32(&m.changed, 1)
+	atomic.StoreInt32(&m.store.changed, 1)
 	return nil
 }
 
@@ -608,8 +419,8 @@ func (m *Document) ReadAll(r io.Reader) error {
 
 		if err := m.readAll(reader); err != nil {
 			if errors.Is(err, io.EOF) || errors.Is(err, io.ErrClosedPipe) || errors.Is(err, os.ErrClosed) {
-				m.offset = m.size
-				atomic.StoreInt32(&m.eof, 1)
+				m.store.offset = m.store.size
+				atomic.StoreInt32(&m.store.eof, 1)
 				return
 			}
 			log.Printf("error: %v\n", err)
@@ -618,3 +429,20 @@ func (m *Document) ReadAll(r io.Reader) error {
 	}()
 	return nil
 }
+
+// readAll reads to the end.
+// The read lines are stored in the lines of the Document.
+func (m *Document) readAll(reader *bufio.Reader) error {
+	chunk := m.store.chunkForAdd(m.seekable, m.store.size)
+	start := len(chunk.lines)
+	for {
+		if err := m.store.readLines(chunk, reader, start, ChunkSize, true); err != nil {
+			return err
+		}
+		chunk = NewChunk(m.store.size)
+		m.store.mu.Lock()
+		m.store.chunks = append(m.store.chunks, chunk)
+		m.store.mu.Unlock()
+		start = 0
+	}
+}
