@@ -256,11 +256,12 @@ func (root *Root) searchMove(ctx context.Context, forward bool, lN int, searcher
 		}
 		root.searchQuit()
 		if err != nil {
-			if lN < root.Doc.startNum {
+			startNum := root.Doc.BufStartNum()
+			if lN < startNum {
 				// If lN is before startNum, move to startNum.
-				root.searchGo(root.Doc.startNum)
-				log.Printf("Moved to %d because %d is out of range.", root.Doc.startNum, lN)
-				return fmt.Errorf("search moved to %d:%w:%v", root.Doc.startNum, err, root.searchWord)
+				root.searchGo(startNum)
+				log.Printf("Moved to %d because %d is out of range.", startNum, lN)
+				return fmt.Errorf("search moved to %d:%w:%v", startNum, err, root.searchWord)
 			}
 			return fmt.Errorf("search:%w:%v", err, root.searchWord)
 		}
@@ -280,16 +281,16 @@ func (m *Document) Search(ctx context.Context, searcher Searcher, chunkNum int,
 	if !m.seekable {
 		m.requestLoad(chunkNum)
 	} else {
-		if m.lastChunkNum() < chunkNum {
+		if m.store.lastChunkNum() < chunkNum {
 			return 0, ErrOutOfChunk
 		}
-		if !m.isLoadedChunk(chunkNum) && !m.storageSearch(ctx, searcher, chunkNum, line) {
+		if !m.store.isLoadedChunk(chunkNum, m.seekable) && !m.storageSearch(ctx, searcher, chunkNum, line) {
 			return 0, ErrNotFound
 		}
 	}
 
 	for n := line; n < ChunkSize; n++ {
-		buf, err := m.GetChunkLine(chunkNum, n)
+		buf, err := m.store.GetChunkLine(chunkNum, n)
 		if err != nil {
 			return n, fmt.Errorf("%w: %d:%d", err, chunkNum, n)
 		}
@@ -307,12 +308,12 @@ func (m *Document) Search(ctx context.Context, searcher Searcher, chunkNum int,
 
 // BackSearch searches backward from the specified line.
 func (m *Document) BackSearch(ctx context.Context, searcher Searcher, chunkNum int, line int) (int, error) {
-	if !m.isLoadedChunk(chunkNum) && !m.storageSearch(ctx, searcher, chunkNum, line) {
+	if !m.store.isLoadedChunk(chunkNum, m.seekable) && !m.storageSearch(ctx, searcher, chunkNum, line) {
 		return 0, ErrNotFound
 	}
 
 	for n := line; n >= 0; n-- {
-		buf, err := m.GetChunkLine(chunkNum, n)
+		buf, err := m.store.GetChunkLine(chunkNum, n)
 		if err != nil {
 			return n, fmt.Errorf("%w: %d:%d", err, chunkNum, n)
 		}
@@ -330,7 +331,7 @@ func (m *Document) BackSearch(ctx context.Context, searcher Searcher, chunkNum i
 
 // storageSearch searches for line not in memory(storage).
 func (m *Document) storageSearch(ctx context.Context, searcher Searcher, chunkNum int, line int) bool {
-	if !m.isLoadedChunk(chunkNum) && atomic.LoadInt32(&m.closed) == 0 {
+	if !m.store.isLoadedChunk(chunkNum, m.seekable) && atomic.LoadInt32(&m.closed) == 0 {
 		if m.requestSearch(chunkNum, searcher) {
 			return true
 		}
@@ -340,8 +341,8 @@ func (m *Document) storageSearch(ctx context.Context, searcher Searcher, chunkNu
 
 // SearchLine searches the document and returns the matching line number.
 func (m *Document) SearchLine(ctx context.Context, searcher Searcher, lN int) (int, error) {
-	lN = max(lN, m.startNum)
-	startChunk, sn := chunkLine(lN)
+	lN = max(lN, m.BufStartNum())
+	startChunk, sn := chunkLineNum(lN)
 
 	for cn := startChunk; ; cn++ {
 		n, err := m.Search(ctx, searcher, cn, sn)
@@ -368,8 +369,8 @@ func (m *Document) SearchLine(ctx context.Context, searcher Searcher, lN int) (i
 // BackSearchLine does a backward search on the document and returns a matching line number.
 func (m *Document) BackSearchLine(ctx context.Context, searcher Searcher, lN int) (int, error) {
 	lN = min(lN, m.BufEndNum()-1)
-	startChunk, sn := chunkLine(lN)
-	minChunk, _ := chunkLine(m.startNum)
+	startChunk, sn := chunkLineNum(lN)
+	minChunk, _ := chunkLineNum(m.BufStartNum())
 	for cn := startChunk; cn >= minChunk; cn-- {
 		n, err := m.BackSearch(ctx, searcher, cn, sn)
 		if err == nil {
@@ -600,7 +601,7 @@ func (root *Root) BackSearch(str string) {
 // searchChunk searches in a Chunk without loading it into memory.
 func (m *Document) searchChunk(chunkNum int, searcher Searcher) (int, error) {
 	// Seek to the start of the chunk.
-	chunk := m.chunks[chunkNum]
+	chunk := m.store.chunks[chunkNum]
 	if _, err := m.file.Seek(chunk.start, io.SeekStart); err != nil {
 		return 0, fmt.Errorf("seek:%w", err)
 	}
