@@ -1,6 +1,7 @@
 package oviewer
 
 import (
+	"regexp"
 	"strings"
 
 	"golang.org/x/exp/constraints"
@@ -32,8 +33,8 @@ func remove[T comparable](list []T, s T) []T {
 	return list
 }
 
-// containsInt returns true if the specified int is included.
-func containsInt(list []int, e int) bool {
+// contains returns true if the specified value is included.
+func contains[T comparable](list []T, e T) bool {
 	for _, n := range list {
 		if e == n {
 			return true
@@ -42,7 +43,29 @@ func containsInt(list []int, e int) bool {
 	return false
 }
 
-// toLast toLast moves the specified string to the end.
+// toAddTop adds the string if it is not in list.
+func toAddTop(list []string, s string) []string {
+	if len(s) == 0 {
+		return list
+	}
+	if !contains(list, s) {
+		list = append([]string{s}, list...)
+	}
+	return list
+}
+
+// toAddLast adds a string to the end if it is not in list.
+func toAddLast(list []string, s string) []string {
+	if len(s) == 0 {
+		return list
+	}
+	if !contains(list, s) {
+		list = append(list, s)
+	}
+	return list
+}
+
+// toLast moves the specified string to the end.
 func toLast(list []string, s string) []string {
 	if len(s) == 0 {
 		return list
@@ -53,8 +76,16 @@ func toLast(list []string, s string) []string {
 	return list
 }
 
-// allIndex returns all matching string positions.
-func allIndex(s string, substr string) [][]int {
+// allIndex is a wrapper that returns either a regular expression index or a string index.
+func allIndex(s string, substr string, reg *regexp.Regexp) [][]int {
+	if reg != nil {
+		return reg.FindAllStringIndex(s, -1)
+	}
+	return allStringIndex(s, substr)
+}
+
+// allStringIndex returns all matching string positions.
+func allStringIndex(s string, substr string) [][]int {
 	if len(substr) == 0 {
 		return nil
 	}
